/*
 * Copyright (c) 2012, Will Strohl
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this 
 * list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice, this 
 * list of conditions and the following disclaimer in the documentation and/or other 
 * materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/

using System.Collections.Generic;
//using System.Xml;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Content;
using DotNetNuke.Entities.Content.Common;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Services.Search;
using System;
using System.Linq;
using System.Text.RegularExpressions;

namespace WillStrohl.Modules.BookmarkCollector.Components
{

    /// -----------------------------------------------------------------------------
    /// <summary>
    /// The Controller class for BookmarkCollector
    /// 
    /// The FeatureController class is defined as the BusinessController in the manifest file (.dnn)
    /// DotNetNuke will poll this class to find out which Interfaces the class implements. 
    /// 
    /// The IPortable interface is used to import/export content from a DNN module
    /// 
    /// The ISearchable interface is used by DNN to index the content of a module
    /// 
    /// The IUpgradeable interface allows module developers to execute code during the upgrade 
    /// process for a module.
    /// 
    /// Below you will find stubbed out implementations of each, uncomment and populate with your own data
    /// </summary>
    /// -----------------------------------------------------------------------------
    public class BookmarkController //: IPortable, ISearchable, IUpgradeable
    {

        #region Constants

        public const string BOOKMARK_MODULE_TYPE_NAME = "WillStrohl.Modules.BookmarkCollector";
        private const string CONTENT_KEY_FORMAT = "bookmarkid={0}";
        private const string CONTENT_KEY_FORMAT_PATTERN = @"^bookmarkid=\d+$";

        #endregion

        #region Private Members

        private int contentTypeId = Null.NullInteger;

        #endregion

        #region Content Item Helpers

        /// <summary>
        /// Returns a ContentTypeID for use in the content item logic in this class
        /// </summary>
        public int BookmarkContentTypeId
        {
            get
            {
                // just return the contentTypeId if it is already populated
                if (contentTypeId != Null.NullInteger) return contentTypeId;

                // create or query for a contentTypeId
                var typeController = new ContentTypeController();
                var colContentTypes =
                    (from t in typeController.GetContentTypes()
                     where t.ContentType == BOOKMARK_MODULE_TYPE_NAME
                     select t);

                // check to see if the content type exists
                if (colContentTypes.Any())
                {
                    // return the existing content type id
                    var contentType = colContentTypes.Single();
                    contentTypeId = contentType == null ? CreateContentType() : contentType.ContentTypeId;
                }
                else
                {
                    // create a content type, then return the new content type id
                    contentTypeId = CreateContentType();
                }

                return contentTypeId;
            }
        }

        #endregion

        #region Data Access

        /// <summary>
        /// This method creates new Bookmark entries in the content item data store
        /// </summary>
        /// <param name="tabId"></param>
        /// <param name="moduleId"></param>
        /// <param name="bookmark"></param>
        /// <returns></returns>
        public int CreateContentItem(int tabId, int moduleId, BookmarkInfo bookmark)
        {
            // create a new content item
            var objContent = new ContentItem
               {
                   Content = JsonExtensionsWeb.ToJsonString(bookmark),
                   ContentTypeId = BookmarkContentTypeId,
                   Indexed = false,
                   ModuleID = moduleId,
                   TabID = tabId
               };

            // save the content item to the database
            objContent.ContentItemId = Util.GetContentController().AddContentItem(objContent);

            // update the objects with the new content item id retrieved in the line above
            bookmark.Id = objContent.ContentItemId;

            // update the content item properties with proper content
            objContent.Content = JsonExtensionsWeb.ToJsonString(bookmark);
            objContent.ContentKey = string.Format(CONTENT_KEY_FORMAT, bookmark.Id);

            // update the content item again since we now have a contentItemId for the properties
            Util.GetContentController().UpdateContentItem(objContent);

            // return the content item id
            return objContent.ContentItemId;
        }

        /// <summary>
        /// This method updates an existing content item
        /// </summary>
        /// <param name="tabId"></param>
        /// <param name="moduleId"></param>
        /// <param name="contentItemId"></param>
        /// <param name="bookmark"></param>
        /// <remarks>This method will only be present in DNN 6.2.3 and newer</remarks>
        public void UpdateContentItem(int tabId, int moduleId, int contentItemId, BookmarkInfo bookmark)
        {
            // get a reference to the original content item
            var objContent = Util.GetContentController().GetContentItem(contentItemId);

            // if the content item doesn't exist, send back to the calling method
            // NOTE:  Probably should just throw an exception here
            if (objContent == null) return;

            // update the relevant properties
            objContent.Content = JsonExtensionsWeb.ToJsonString(bookmark);

            // these are just for data posterity and probably don't need to be done
            objContent.ModuleID = moduleId;  
            objContent.TabID = tabId;

            // data integrity check for legacy data (just in case stuff)
            if (!Regex.IsMatch(objContent.ContentKey, CONTENT_KEY_FORMAT_PATTERN, RegexOptions.IgnoreCase))
            {
                objContent.ContentKey = string.Format(CONTENT_KEY_FORMAT, objContent.ContentItemId);
            }
            
            // execute the update on the database
            Util.GetContentController().UpdateContentItem(objContent);
        }

        /// <summary>
        /// This method deletes a content item
        /// </summary>
        /// <param name="contentItemId"></param>
        public void DeleteContentItem(int contentItemId)
        {
            // execute the deletion
            Util.GetContentController().DeleteContentItem(contentItemId);
        }

        /// <summary>
        /// This method deletes a content item
        /// </summary>
        /// <param name="contentItem"></param>
        public void DeleteContentItem(ContentItem contentItem)
        {
            // execute the deletion
            Util.GetContentController().DeleteContentItem(contentItem);
        }

        /// <summary>
        /// This method retrieves a content item from the database
        /// </summary>
        /// <param name="contentItemId"></param>
        /// <returns></returns>
        public BookmarkInfo GetContentItem(int contentItemId)
        {
            // get a reference to the content item
            var objContent = Util.GetContentController().GetContentItem(contentItemId);

            // parse the content item and return a bookmark object
            return ParseContentItemForBookmark(objContent);
        }

        /// <summary>
        /// This method retrieves a list of content items from the database
        /// </summary>
        /// <param name="moduleId"></param>
        /// <returns></returns>
        public IList<BookmarkInfo> GetContentItems(int moduleId)
        {
            // get a collection of content items for this module that match this content type id
            var collContent = (
                from t in Util.GetContentController().GetContentItemsByModuleId(moduleId)
                where t.ContentTypeId == BookmarkContentTypeId
                orderby t.Content
                select t);

            //
            // NOTE:  The above is necessary because there could be multiple content types 
            // for a single module, for things like taxonomy and any number of other things 
            // that might be extending content items.
            //

            // parse the content item collection and return a bookmark collection
            return ParseContentItemsForBookmarkCollection(collContent);
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// Parse the content item content to populate a bookmark object
        /// </summary>
        /// <param name="contentItem"></param>
        /// <returns></returns>
        private BookmarkInfo ParseContentItemForBookmark(ContentItem contentItem)
        {
            var bookmark = new BookmarkInfo();
            if (contentItem != null && !string.IsNullOrEmpty(contentItem.Content))
            {
                bookmark = contentItem.Content.FromJson<BookmarkInfo>();
            }

            return bookmark;
        }

        /// <summary>
        /// Iterates through a collection of content items and parses them each to popular a bookmark collection
        /// </summary>
        /// <param name="contentItems"></param>
        /// <returns></returns>
        private IList<BookmarkInfo> ParseContentItemsForBookmarkCollection(IQueryable<ContentItem> contentItems)
        {
            var bookmarks = new List<BookmarkInfo>(contentItems.Count());

            foreach (ContentItem contentItem in contentItems)
            {
                BookmarkInfo bookmark = ParseContentItemForBookmark(contentItem);
                bookmarks.Add(bookmark);
            }

            return bookmarks;
        }

        /// <summary>
        /// Creates a unique content type for this module
        /// </summary>
        /// <returns></returns>
        private static int CreateContentType()
        {
            var typeController = new ContentTypeController();
            var objContentType = new ContentType { ContentType = BOOKMARK_MODULE_TYPE_NAME };

            return typeController.AddContentType(objContentType);
        }

        #endregion

        #region Optional Interfaces

        /// -----------------------------------------------------------------------------
        /// <summary>
        /// ExportModule implements the IPortable ExportModule Interface
        /// </summary>
        /// <param name="ModuleID">The Id of the module to be exported</param>
        /// -----------------------------------------------------------------------------
        //public string ExportModule(int ModuleID)
        //{
        //string strXML = "";

        //List<BookmarkCollectorInfo> colBookmarkCollectors = GetBookmarkCollectors(ModuleID);
        //if (colBookmarkCollectors.Count != 0)
        //{
        //    strXML += "<BookmarkCollectors>";

        //    foreach (BookmarkCollectorInfo objBookmarkCollector in colBookmarkCollectors)
        //    {
        //        strXML += "<BookmarkCollector>";
        //        strXML += "<content>" + DotNetNuke.Common.Utilities.XmlUtils.XMLEncode(objBookmarkCollector.Content) + "</content>";
        //        strXML += "</BookmarkCollector>";
        //    }
        //    strXML += "</BookmarkCollectors>";
        //}

        //return strXML;

        //	throw new System.NotImplementedException("The method or operation is not implemented.");
        //}

        /// -----------------------------------------------------------------------------
        /// <summary>
        /// ImportModule implements the IPortable ImportModule Interface
        /// </summary>
        /// <param name="ModuleID">The Id of the module to be imported</param>
        /// <param name="Content">The content to be imported</param>
        /// <param name="Version">The version of the module to be imported</param>
        /// <param name="UserId">The Id of the user performing the import</param>
        /// -----------------------------------------------------------------------------
        //public void ImportModule(int ModuleID, string Content, string Version, int UserID)
        //{
        //XmlNode xmlBookmarkCollectors = DotNetNuke.Common.Globals.GetContent(Content, "BookmarkCollectors");
        //foreach (XmlNode xmlBookmarkCollector in xmlBookmarkCollectors.SelectNodes("BookmarkCollector"))
        //{
        //    BookmarkCollectorInfo objBookmarkCollector = new BookmarkCollectorInfo();
        //    objBookmarkCollector.ModuleId = ModuleID;
        //    objBookmarkCollector.Content = xmlBookmarkCollector.SelectSingleNode("content").InnerText;
        //    objBookmarkCollector.CreatedByUser = UserID;
        //    AddBookmarkCollector(objBookmarkCollector);
        //}

        //	throw new System.NotImplementedException("The method or operation is not implemented.");
        //}

        /// -----------------------------------------------------------------------------
        /// <summary>
        /// GetSearchItems implements the ISearchable Interface
        /// </summary>
        /// <param name="ModInfo">The ModuleInfo for the module to be Indexed</param>
        /// -----------------------------------------------------------------------------
        //public DotNetNuke.Services.Search.SearchItemInfoCollection GetSearchItems(DotNetNuke.Entities.Modules.ModuleInfo ModInfo)
        //{
        //SearchItemInfoCollection SearchItemCollection = new SearchItemInfoCollection();

        //List<BookmarkCollectorInfo> colBookmarkCollectors = GetBookmarkCollectors(ModInfo.ModuleID);

        //foreach (BookmarkCollectorInfo objBookmarkCollector in colBookmarkCollectors)
        //{
        //    SearchItemInfo SearchItem = new SearchItemInfo(ModInfo.ModuleTitle, objBookmarkCollector.Content, objBookmarkCollector.CreatedByUser, objBookmarkCollector.CreatedDate, ModInfo.ModuleID, objBookmarkCollector.ItemId.ToString(), objBookmarkCollector.Content, "ItemId=" + objBookmarkCollector.ItemId.ToString());
        //    SearchItemCollection.Add(SearchItem);
        //}

        //return SearchItemCollection;

        //	throw new System.NotImplementedException("The method or operation is not implemented.");
        //}

        /// -----------------------------------------------------------------------------
        /// <summary>
        /// UpgradeModule implements the IUpgradeable Interface
        /// </summary>
        /// <param name="Version">The current version of the module</param>
        /// -----------------------------------------------------------------------------
        //public string UpgradeModule(string Version)
        //{
        //	throw new System.NotImplementedException("The method or operation is not implemented.");
        //}

        #endregion

    }

}
